@@ -502,7 +502,6 @@ public:
     template<class T> void BinWrite (const T &x){
         for(unsigned int i=0;i<sizeof(T)/2;i++)
             Write((reinterpret_cast<const unsigned short *>(&x))[i]);
-        return *this;
     }
 
 bool End(){return readOut>=static_cast<unsigned int>(data.Len());}
@@ -514,7 +513,6 @@ bool End(){return readOut>=static_cast<unsigned int>(data.Len());}
     template<class T> void BinRead (const T &x){
         for(unsigned int i=0;i<sizeof(T)/2;i++)
             Read(reinterpret_cast<unsigned short *>(&x)[i]);
-        return *this;
     }
 
 
@@ -596,7 +594,7 @@ public:
     nWaitForAck(nMessage* m,int rec);
     virtual ~nWaitForAck();
 
-    virtual void AckExtraAction(){};
+    virtual void AckExtraAction(){}
 
     static void Ackt(unsigned short id,unsigned short peer);
 
