@@ -33,11 +33,12 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #include "config.h"
 #include "tRandom.h"
-
+#include "tSysTime.h"
 #include "tRandom.h"
 
 #include <string>
 #include <stdio.h>
+#include <string.h>
 
 #include <sys/types.h>
 
@@ -47,6 +48,9 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #ifndef WIN32
 #include <arpa/inet.h> 
+#include <netinet/in_systm.h>
+#include <netinet/in.h>
+#include <netinet/ip.h>
 #include <netdb.h>
 #include <sys/param.h>
 #include <sys/ioctl.h>
@@ -650,7 +654,7 @@ public:
 bool ANET_ListenOn( char * hostname, int net_hostport,  nSocketListener::SocketArray & sockets, bool & onesocketonly, bool & relaxed, bool & ignoreErrors )
 {
     // skip whitespace
-    while ( isspace( *hostname ) )
+    while ( isblank( *hostname ) )
         hostname ++;
 
     // quit silently on empty hostname
@@ -692,7 +696,7 @@ bool ANET_Listen (bool state, int net_hostport, const tString & net_hostip, nSoc
         int lastpos = 0;            // position where the current IP starts
         for ( int pos = 0; pos < ips.Len(); ++pos )
         {
-            if (isspace(ips[pos]))
+            if (isblank(ips[pos]))
             {
                 ips[pos] = '\0';
                 if ( pos > lastpos )
@@ -1403,6 +1407,22 @@ const nAddress & nAddress::GetPort( int & port ) const
     return *this;
 }
 
+
+// *******************************************************************************************
+// *
+// *	IsSet
+// *
+// *******************************************************************************************
+//!
+//!		@return		    true only if address was set to something valid
+//!
+// *******************************************************************************************
+
+bool nAddress::IsSet () const
+{
+    return addr_.addr_in.sin_addr.s_addr != INADDR_ANY;
+}
+
 // *******************************************************************************************
 // *
 // *	Compare
@@ -1501,6 +1521,14 @@ int nSocket::Create( void )
     if ( socket_ < 0 )
         return -1;
 
+    // set TOS to low latency ( see manpages getsockopt(2), ip(7) and socket(7) )
+    // maybe this works for Windows, too?
+#ifndef WIN32
+    char tos = IPTOS_LOWDELAY;
+
+    setsockopt( socket_, IPPROTO_IP, IP_TOS, &tos, sizeof(char) );
+#endif    
+
     // unblock it
     bool _true = true;
     return ioctl (socket_, FIONBIO, reinterpret_cast<char *>(&_true)) == -1;
@@ -2001,6 +2029,7 @@ int nSocket::Read( int8 * buf, int len, nAddress & addr ) const
 
     if ( ret >= 0 )
     {
+        // con << "Read " << ret << " bytes from socket " << GetSocket() << ".\n";
     }
 
 #ifdef PRINTPACKETS
@@ -2579,6 +2608,64 @@ nSocket * nBasicNetworkSystem::Init()
 
 }
 
+// *******************************************************************************
+// *
+// *	Select
+// *
+// *******************************************************************************
+//!
+//!		@param	dt	the time in seconds to wait at max
+//!		@return		true if data came in
+//!
+// *******************************************************************************
+
+bool nBasicNetworkSystem::Select( REAL dt )
+{
+    int retval = 0;
+    static char const * section = "NETSELECT";
+    if ( !tRecorder::PlaybackStrict( section, retval ) )
+    {
+        if ( controlSocket_.GetSocket() < 0 )
+        {
+            tDelay( int( dt * 1000000 ) );
+            return false;
+        }
+
+        fd_set rfds; // set of sockets to wathc
+        struct timeval tv; // time value to pass to select()
+
+        FD_ZERO( &rfds );
+
+        // watch the control socket
+        FD_SET( controlSocket_.GetSocket(), &rfds );
+        // con << "Watching " << controlSocket_.GetSocket();
+
+        int max = controlSocket_.GetSocket();
+
+        // watch listening sockets
+        for( nSocketListener::SocketArray::const_iterator iter = listener_.GetSockets().begin(); iter != listener_.GetSockets().end(); ++iter )
+        {
+            FD_SET( (*iter).GetSocket(), &rfds );
+            if ( (*iter).GetSocket() > max )
+                max = (*iter).GetSocket();
+            // con << ", " << (*iter).GetSocket();
+        }
+
+        // set time
+        tv.tv_sec  = static_cast< long int >( dt );
+        tv.tv_usec = static_cast< long int >( (dt-tv.tv_sec)*1000000 );
+
+        // delegate to system select
+        retval = select(max+1, &rfds, NULL, NULL, &tv);
+    }
+    tRecorder::Record( section, retval );
+
+    // con << " : " << retval << "\n";
+
+    // return result
+    return ( retval > 0 );
+}
+
 // *******************************************************************************************
 // *
 // *	Shutdown
