@@ -20,7 +20,7 @@ GNU General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-  
+
 ***************************************************************************
 
 */
@@ -45,6 +45,9 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "tRecorder.h"
 
 #ifndef DEDICATED
+#include "SDL_thread.h"
+#include "SDL_mutex.h"
+
 #include <png.h>
 #define SCREENSHOT_PNG_BITDEPTH 8
 #define SCREENSHOT_BYTES_PER_PIXEL 3
@@ -85,7 +88,7 @@ static GLXContext cx;
 Display *dpy=NULL;
 Window  win;
 
-#endif 
+#endif
 
 #ifdef DIRTY
 #include <SDL_syswm.h>
@@ -112,15 +115,15 @@ bool  rSysDep::InitGL(){
         std::cerr << "No 3Dfx hardware available.\n" << x << '\n';
         return(false);
       }
-      
+
       GLint attribs[]={FXMESA_DOUBLEBUFFER,FXMESA_DEPTH_SIZE,16,FXMESA_NONE};
       ctx=fxMesaCreateBestContext(0,sr_screenWidth,sr_screenHeight,attribs);
-      
+
       if (!ctx){
         std::cerr << "Could not create FX rendering context!\n";
         return(false);
       }
-      
+
       fxMesaMakeCurrent(ctx);
     }
     */
@@ -199,7 +202,7 @@ bool  rSysDep::InitGL(){
         }
     }
 
-#endif 
+#endif
 
     return true;
 }
@@ -240,7 +243,7 @@ void  rSysDep::ExitGL(){
         glXDestroyContext(dpy, cx );
         dpy=NULL;
     }
-#endif 
+#endif
 }
 #endif // DIRTY
 
@@ -454,14 +457,14 @@ static void sr_DelayFrame( int targetFPS )
     double thisFrame = tRealSysTimeFloat();
 
     int uSecsPassed = static_cast<int>( MILLION * ( thisFrame - lastFrame ) );
-    
+
 //    con << uSecsPassed << "\n";
 
     // wait
     int uSecsToWait = uSecsPerFrame - uSecsPassed;
     if ( uSecsToWait > 0 )
         tDelay( uSecsToWait );
-    
+
     // call glFinish to wait for GPU
     glFinish();
 }
@@ -471,10 +474,92 @@ rSysDep::rSwapMode rSysDep::swapMode_ = rSysDep::rSwap_glFlush;
 //rSysDep::rSwapMode rSysDep::swapMode_ = rSysDep::rSwap_60Hz;
 
 // buffer swap:
-#ifndef DEDICATED   
+#ifndef DEDICATED
 // for setting breakpoints in optimized mode, too
 static void breakpoint(){}
 
+static bool sr_netSyncThreadGoOn = true;
+static rSysDep::rNetIdler * sr_netIdler = NULL;
+int sr_NetSyncThread(void *lockVoid)
+{
+    SDL_mutex *lock = (SDL_mutex *)lockVoid;
+
+    SDL_mutexP(lock);
+
+    while ( sr_netSyncThreadGoOn )
+    {
+        SDL_mutexV(lock);
+        // wait for network data
+        bool toDo = sr_netIdler->Wait();
+        SDL_mutexP(lock);
+
+        if ( toDo )
+        {
+            // disable rendering (during auto-scrolling of console, for example)
+            bool glout = sr_glOut;
+            sr_glOut = false;
+
+            // new network data arrived, handle it
+            sr_netIdler->Do();
+
+            // enable rendering again
+            sr_glOut = glout;
+        }
+    }
+
+    SDL_mutexV(lock);
+
+    return 0;
+}
+
+static SDL_Thread * sr_netSyncThread = NULL;
+static SDL_mutex * sr_netLock = NULL;
+void rSysDep::StartNetSyncThread( rNetIdler * idler )
+{
+    sr_netIdler = idler;
+
+    return;
+
+    // can't use thrading trouble while recording
+    if ( tRecorder::IsRunning() )
+        return;
+
+    if ( sr_netSyncThread )
+        return;
+
+    // create lock
+    if ( !sr_netLock )
+        sr_netLock = SDL_CreateMutex();
+
+    // start thread
+    sr_netSyncThread = SDL_CreateThread( sr_NetSyncThread, sr_netLock );
+    if ( !sr_netSyncThread )
+        return;
+
+    // lock mutex, the thread should only do work while the main thread is waiting for the refresh
+    SDL_mutexP( sr_netLock );
+}
+
+void rSysDep::StopNetSyncThread()
+{
+    // stop and delete thread
+    if ( sr_netSyncThread )
+    {
+        SDL_mutexV(  sr_netLock );
+        sr_netSyncThreadGoOn = false;
+        SDL_WaitThread( sr_netSyncThread, NULL );
+        sr_netSyncThread = NULL;
+        sr_netIdler = NULL;
+    }
+
+    // delete lock
+    if ( sr_netLock )
+    {
+        SDL_DestroyMutex( sr_netLock );
+        sr_netLock = NULL;
+    }
+}
+
 void rSysDep::SwapGL(){
     if ( s_benchmark )
     {
@@ -537,18 +622,26 @@ void rSysDep::SwapGL(){
     if (!sr_glOut)
     {
         // display next frame in fast foward mode
-        if ( s_fastForward && ( time > s_nextFastForwardFrameRecorded || realTime > s_nextFastForwardFrameReal ) || next_glOut )
+        if ( ( s_fastForward && ( time > s_nextFastForwardFrameRecorded || realTime > s_nextFastForwardFrameReal ) ) || next_glOut )
         {
             sr_glOut = true;
             rSysDep::ClearGL();
         }
+
+        // in playback or recording mode, always execute frame tasks, they may be improtant for consistency
+        if ( tRecorder::IsRunning() )
+            rPerFrameTask::DoPerFrameTasks();
+
         return;
     }
 
-    sr_LockSDL();
 
     rPerFrameTask::DoPerFrameTasks();
 
+    // unlock the mutex while waiting for the swap operation to finish
+    SDL_mutexV(  sr_netLock );
+    sr_LockSDL();
+
     switch( swapMode_ )
     {
     case rSwap_Fastest:
@@ -584,6 +677,9 @@ void rSysDep::SwapGL(){
     }
 
     sr_UnlockSDL();
+    // lock mutex again
+    SDL_mutexP(  sr_netLock );
+
 
     // disable output in fast forward mode
     if ( s_fastForward && tRecorder::IsPlayingBack() )
