@@ -27,6 +27,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #include "tMemManager.h"
 #include "tString.h"
+#include "tColor.h"
 #include "tLocale.h"
 #include "tConfiguration.h"
 #include <ctype.h>
@@ -69,6 +70,13 @@ static bool st_ReadEscapeSequence( char & c, char & c2, std::istream & s )
     {
         c = s.get();
 
+        // nothing useful read?
+        if ( s.eof() )
+        {
+            c = '\\';
+            return false;
+        }
+
         // interpret special escape sequences
         switch (c)
         {
@@ -77,7 +85,9 @@ static bool st_ReadEscapeSequence( char & c, char & c2, std::istream & s )
             c = '\n';
             return true;
         case '"':
+        case ' ':
         case '\'':
+        case '\\':
         case '\n':
             // include quoting character as literal
             return true;
@@ -99,8 +109,10 @@ static bool st_ReadEscapeSequence( char & c, std::istream & s )
 {
     char c2 = '\0';
     bool ret = st_ReadEscapeSequence( c, c2, s );
-    if ( c2 )
+    if ( c2 && s.good() )
+    {
         s.putback( c2 );
+    }
     return ret;
 }
 
@@ -108,15 +120,26 @@ void tString::ReadLine(std::istream &s, bool enableEscapeSequences ){
     char c=' ';
     int i=0;
     SetLen(0);
-    while(c!='\n' && c!='\r' && isspace(c) &&  s.good() && !s.eof()){
+
+    // note: this is not the same as std::ws(s), we're stopping at newlines.
+    while(c!='\n' && c!='\r' && isblank(c) &&  s.good() && !s.eof()){
         c=s.get();
     }
+    if(s.good())
+    {
+        s.putback(c);
+    }
 
-    s.putback(c);
     c='x';
 
-    while(c!='\n' && c!='\r' && s.good() && !s.eof()){
+    while( true )
+    {
         c=s.get();
+
+        // notice end of line or file
+        if ( c=='\n' || c=='\r' || !s.good() || s.eof())
+            break;
+
         if ( enableEscapeSequences )
         {
             char c2 = '\0';
@@ -136,10 +159,8 @@ void tString::ReadLine(std::istream &s, bool enableEscapeSequences ){
         }
         operator[](i++)=c;
     }
-    if (i>0)
-        operator[](i-1)='\0';
-    else
-        operator[](0)='\0';
+
+    operator[](i)='\0';
 }
 
 tString & tString::operator<<(const char *c){
@@ -215,11 +236,13 @@ std::ostream & operator<< (std::ostream &s,const tString &x){
 }
 
 std::istream & operator>> (std::istream &s,tString &x){
-    char c=' ';
     int i=0;
     x.SetLen(0);
 
-    while(isspace(c) && s.good() && !s.eof()) c=s.get();
+    // eat whitespace
+    std::ws(s);
+
+    char c=s.get();
 
     // check if the string is quoted
     bool quoted = false;
@@ -231,28 +254,68 @@ std::istream & operator>> (std::istream &s,tString &x){
         c = s.get();
     }
 
-    while((quoted || !isspace(c)) && s.good() && !s.eof()){
-        x[i++]=c;
-        c=s.get();
-
+    bool lastEscape = false;
+    while((quoted || !( isblank(c) || c == '\n' || c == '\r') ) && s.good() && !s.eof()){
         // read and interpret escape sequences
-        if ( !st_ReadEscapeSequence( c, s) )
+        bool thisEscape = false;
+        if ( !lastEscape && !( thisEscape = st_ReadEscapeSequence( c, s ) ) && quoted && c == quoteChar )
         {
-            // interpret special characters
-            if ( quoted && c == quoteChar )
-            {
-                // this marks the end of a quoted string; abort.
-                c = s.get();
-                break;
-            }
+            // no escape, this string is quoted and the current character is an end quote. We're finished.
+            c = s.get();
+            break;
+        }
+        else
+        {
+            // append escaped or regular character
+            x[i++]=c;
+            c=s.get();
         }
 
+        // lastEscape = thisEscape;
+    }
+
+    if(s.good())
+    {
+        s.putback(c);
     }
-    s.putback(c);
     x[i]='\0';
     return s;
 }
 
+#ifdef DEBUG
+class tQuoteTester
+{
+    // checks that a specific input produces a specific output when read via <<
+    static void Test( char const * in, char const * out )
+    {
+        std::stringstream s;
+        s << in << " NEXTWORD";
+        tString o;
+        s >> o;
+        tASSERT( o == out );
+    }
+
+public:
+    tQuoteTester()
+    {
+        // basic stuff
+        Test( "abc", "abc" );
+        
+        // quoted
+        Test( "'abc'", "abc" );
+
+        // escaped
+        Test( "\\'a\\\"b", "'a\"b" );
+        
+        // chained backslashes
+        Test( "\\\\", "\\" );
+        Test( "\\\\\\\\", "\\\\" );
+        Test( "'\\\\\\\\'", "\\\\" );
+    }
+};
+
+static tQuoteTester tester;
+#endif
 
 // *******************************************************************************************
 // *
@@ -286,6 +349,10 @@ void tString::SetPos(int l, bool cut){
             operator+=(' ');
         }
     }
+    if( l == Len() && !cut)
+    {
+        operator+=(' ');
+    }
     for(i=Len();i<l;i++)
         operator+=(' ');
 }
@@ -369,7 +436,7 @@ int tString::StrPos( const tString &tofind ) const
     for (int i=0; i<Len()-1; i++) {
         if ((*this)(i) == tofind(0)) {
             bool found = true;
-            for (int j=0; j<tofind.Len()-1; j++) {
+            for (int j=0; j<tofind.Len()-1 && i+j < Len()-1; j++) {
                 if ((*this)(i+j) != tofind(j))
                     found = false;
             }
@@ -421,14 +488,13 @@ int tString::toInt( int pos ) const
         pos++;
     }
 
-    if ( ret > 0 )
-    {
-        return ret;
-    }
-    else
+    // check whether the last character read was not whitespace or the end of the string
+    if ( pos < Len() && !isblank((*this)(pos-1)) )
     {
         return 0;
     }
+
+    return ret;
 }
 
 int tString::toInt() const {
@@ -1050,6 +1116,65 @@ tString & tColoredString::operator =( const tOutput & s )
     return *this;
 }
 
+// *******************************************************************************************
+// *
+// *	RemoveColors
+// *
+// *******************************************************************************************
+//!
+//!		@param	c	C style string or tString to clear of color codes
+//!		@return   	cleared string
+//!
+// *******************************************************************************************
+
+tString tColoredString::RemoveColors( const char * c, bool darkonly )
+{
+    // st_Breakpoint();
+    tString ret;
+    int len = strlen(c);
+    bool removed = false;
+    
+    // walk through string
+    while (*c!='\0'){
+        // skip color codes
+        if (*c=='0' && len >= 2 && c[1]=='x')
+        {
+            if( len >= 8 && darkonly )
+            {
+                tColor colorToFilter( c );
+                if ( !colorToFilter.IsDark() || strncmp( c, "0xRESETT", 8 ) == 0 )
+                    ret << c[0] << c[1] << c[2] << c[3] << c[4] << c[5] << c[6] << c[7];
+                else
+                    removed = true;
+
+                c   += 8;
+                len -= 8;	
+            }
+            else if( len >= 8 )
+            {
+                c   += 8;
+                len -= 8;
+                removed = true;
+            }
+            else
+            {
+                // skip incomplete color codes, too
+                return RemoveColors( ret, darkonly );
+            }
+            // st_Breakpoint();
+        }
+        else
+        {
+            ret << (*(c++));
+            len--;
+        }
+    }
+    
+    // st_Breakpoint();
+    
+    return removed ? RemoveColors( ret, darkonly ) : ret;
+}
+
 // *******************************************************************************************
 // *
 // *	RemoveColors
@@ -1063,19 +1188,7 @@ tString & tColoredString::operator =( const tOutput & s )
 
 tString tColoredString::RemoveColors( const char * c )
 {
-    tString ret;
-
-    // walk through string
-    while (*c!='\0'){
-        // skip color codes
-        if (*c=='0' && strlen(c)>=8 && c[1]=='x')
-        {
-            c+=8;
-        }
-        else
-            ret << (*(c++));
-    }
-    return ret;
+    return RemoveColors ( c, false );
 }
 
 // helper function: removes trailing color of string and returns number of chars
@@ -1231,7 +1344,7 @@ private:
 //!
 // *******************************************************************************************
 
-void tColoredString::NetFilter( void )
+void tString::NetFilter( void )
 {
     static tCharacterFilter filter;
 
@@ -1321,6 +1434,12 @@ tColoredString & operator <<(tColoredString &s, const tColoredStringProxy &color
 {
     if (st_colorStrings)
     {
+        if ( colorCode.r_ == -1 || colorCode.g_ == -1 || colorCode.b_ == -1 )
+        {
+            s << "0xRESETT";
+            return s;
+        }
+
         char cs[9];
         cs[0]='0';
         cs[1]='x';
@@ -1354,13 +1473,13 @@ tColoredString & operator <<(tColoredString &s, const tColoredStringProxy &color
 //!
 // *******************************************************************************************
 
-tString tString::StripWhitespace( void )
+tString tString::StripWhitespace( void ) const
 {
     tString toReturn;
 
     for( int i = 0; i<=Len()-2; i++ )
     {
-        if( !isspace((*this)(i)) )
+        if( !isblank((*this)(i)) )
             toReturn << (*this)(i);
     }
     return toReturn;
@@ -1536,3 +1655,108 @@ tString tString::Truncate( int truncateAt ) const
 
     return SubStr( 0, truncateAt ) << "...";
 }
+
+// *******************************************************************************************
+// *
+// *	tIsInList
+// *
+// *******************************************************************************************
+//!
+//!    @param      list       The string representation of the list
+//!    @param      item       The item to look for
+//!    @return     true if the list contains the item
+//!
+//!    Example: tIsInList( "ab, cd", "ab" ) -> true
+//!
+// *******************************************************************************************
+//! check whether item is in a comma or whitespace separated list
+bool tIsInList( tString const & list_, tString const & item )
+{
+    tString list = list_;
+
+    while( list != "" )
+    {
+        // find the item
+        int pos = list.StrPos( item );
+
+        // no traditional match? shoot.
+        if ( pos < 0 )
+        {
+            return false;
+        }
+
+        // check whether the match is a true list match
+        if ( 
+            ( pos == 0 || list[pos-1] == ',' || isblank(list[pos-1]) )
+            &&
+            ( pos + item.Len() >= list.Len() || list[pos+item.Len()-1] == ',' || isblank(list[pos+item.Len()-1]) )
+            )
+        {
+            return true;
+        }
+        else
+        {
+            // no? truncate the list and go on.
+            list = list.SubStr( pos + 1 );
+        }
+    }
+    
+    return false;
+}
+
+// **********************************************************************
+// *
+// *	tToLower
+// *
+// **********************************************************************
+//!
+//!    @param      toTransform   The string to transform
+//!
+// **********************************************************************
+void tToLower( tString & toTransform )
+{
+    for( int i = toTransform.Len()-2; i >= 0; --i )
+    {
+        toTransform[i] = tolower( toTransform[i] );
+    }
+}
+
+// **********************************************************************
+// *
+// *	tToUpper
+// *
+// **********************************************************************
+//!
+//!    @param      toTransform   The string to transform
+//!
+// **********************************************************************
+void tToUpper( tString & toTransform )
+{
+    for( int i = toTransform.Len()-2; i >= 0; --i )
+    {
+        toTransform[i] = toupper( toTransform[i] );
+    }
+}
+
+tString st_GetCurrentTime( char const * szFormat )
+{
+    char szTemp[128];
+    time_t     now;
+    struct tm *pTime;
+    now = time(NULL);
+    pTime = localtime(&now);
+    strftime(szTemp,sizeof(szTemp),szFormat,pTime);
+    return tString(szTemp);
+}
+
+// replacement for tString::EndsWith from the trunk
+bool st_StringEndsWith( tString const & test, tString const & end )
+{
+    int start = test.Len() - end.Len();
+    return start >= 0 && test.SubStr( start ) == end;
+}
+
+bool st_StringEndsWith( tString const & test, char const * end )
+{
+    return st_StringEndsWith( test, tString( end ) );
+}
